%
% f=fun(sigma)
%
% Function f=fun(sigma) computes the regularized objective
% function for the EM-38 inversion problem.  
% 
% Input Parameters:
%   sigma - array of conductivities per layer
%
% Output Parameters:
%   f - vector of forward model predictions minus data and alpha*L*m.
%
% Globals:
%   HDATA       vector of measured data in horizontal mode.
%   HHEIGHT     heights for vdata.
%   VDATA       vector of measured data in vertical mode.
%   VHEIGHT     heights for vdata.
%   ALPHA       regularization parameter.
%   L           smoothing matrix.
%   M           number of layers.
%   D           layer thicknesses.
%
function fvec=fun(sigma)
%
% Declare required globals.
%
global VHEIGHT;
global VDATA;
global HHEIGHT;
global HDATA;
global ALPHA;
global L;
global M;
global D;
%
% Operating frequency.
%
f=14600;
OMEGA=2*pi*f;
%
% Distance between coils.
%
R=1.0;
%
% Conductivity of the air above the top layer.
%
SIGMA0=0;
%
% Magnetic permeabilities.  
%
MU0=pi*4e-7;
MU=MU0*ones(M,1);
%
% Scaling factor Delta.
%
DELTA=sqrt(2/(sigma(1)*MU0*OMEGA));
%
% Figure out the size parameters, and setup the f vector.
%
[nrows,ncols]=size(L);
fvec=zeros(length(VDATA)+length(HDATA)+nrows,1);
%
% Now, do the predictions.
%
for i=1:length(VHEIGHT)
  H=VHEIGHT(i);
  [predv,predh]=predict(R,DELTA,H,M,MU,MU0,sigma,SIGMA0,D,OMEGA);
  fvec(i)=predv-VDATA(i);
end
for i=1:length(HHEIGHT)
  H=HHEIGHT(i);
  [predv,predh]=predict(R,DELTA,H,M,MU,MU0,sigma,SIGMA0,D,OMEGA);
  fvec(i+length(VDATA))=predh-HDATA(i);
end
%
% Add in the regularization terms.
%
fvec((length(VDATA)+length(HDATA)+1):end)=ALPHA*L*sigma;


