%
% This script solves the inverse conductivity estimation problem
% for the data in site1.mat.  This data set was the first example
% in the paper:
%
%  B. Borchers, T. Uram, and J. M. H. Hendrickx.  Tikhonov
%  Regularization of Electrical Conductivity Depth Profiles in
%  Field Soils.  Soil Science Society of America Journal
%  61:1004-1009, 1997.
%
% The solution obtained is very close to the solution shown in
% figure 3 of this paper.
%
%
% Setup global variables.
%
% Globals:
%   HDATA       vector of measured data in horizontal mode.
%   HHEIGHT     heights for vdata.
%   VDATA       vector of measured data in vertical mode.
%   VHEIGHT     heights for vdata.
%   ALPHA       regularization parameter.
%   L           smoothing matrix.
%   M           number of layers.
%   D           layer thicknesses.
%
global VHEIGHT;
global VDATA;
global HHEIGHT;
global HDATA;
global ALPHA;
global L;
global M;
global D;
%
% load in the data and copy from the site1.mat variables into the
% global variables.  This would obviously have to change for other
% data sets.
%
load site1.mat
VHEIGHT=vheight;
VDATA=vdata;
HHEIGHT=hheight;
HDATA=hdata;
%
% Setup the layering.  We'll use 15 layers of 0.1m followed by a
% semiinfinite layer.  
%
M=16;
D=0.1*ones(M,1);
%
% Setup the L matrix.  See (13) in the paper referenced above. 
%
e=ones(M-1,1);
L=spdiags([e -2*e e],-1:1,M-1,M);
L=L(2:end,:);
%
% Pick a range of regularization parameters to test.  These values
% were picked by trial and error to cover the range from a very
% underregularized solution to a very overregularized solution.
%
alphas=[1; 3; 5; 7; 10; 30;  50; 70;  100; 300;  500; ...
	700; 1000; 3000;  5000; 7000; 10000];
%
% Loop through the ranges of alphas, solving each regularized problem.
% For the largest value of alpha, we start with a simple constant
% model of 100 mS/m.  After that, we go through the alphas one at a
% time from largest to smallest, using the previous solution as the
% starting point for the next call to lm.  This helps to speed
% convergence and avoid local minimum.  
%
% Note that we're using a specially modified version of the
% Levenberg-Marquardt routine in order to avoid negative
% conductivities, which make no physical sense.
%
% Also note that the finite difference derivatives used here are
% inaccurate enough that we some times have trouble reaching a
% solution with the desired accuracy.  If nothing else, we'll stop
% after maxiter=50 iterations.
%
for k=length(alphas):-1:1
  ALPHA=alphas(k)
  if (k==length(alphas))
    m0=0.1*ones(M,1);
  else
    m0=mstar(:,k+1);
  end
  mstar(:,k)=lm('fun','jac',m0,1.0e-6,50);
end
%
% Compute the l-curve
%
for k=1:length(alphas)
  normlm(k)=norm(L*mstar(:,k)); 
  f=fun(mstar(:,k));
  normgmd(k)=norm(f(1:(length(VDATA)+length(HDATA))));
end
%
% Plot the L-curve.
%
figure(1);
loglog(normgmd,normlm,'ko-');
xlabel('norm(G(m)-d)');
ylabel('norm(L*m)');
print -dpdf fig1.pdf
%
% The corrner of the L-curve occurs near alphas(6)=30.  Actually
% several values of alpha near 30 give essentially the same
% solution.  
%
mbest=mstar(:,6);
%
% Plot the best solution.
%
figure(2);
plotconst(1000*mbest,0,1.6);
xlabel('Depth (m)');
ylabel('Conductivity (mS/m)');
print -dpdf fig2.pdf
